/***********************************************************************
 *
 * NetSaint CGI Utilities
 * 
 * Written By: Ethan Galstad (netsaint@linuxbox.com)
 * Last Modified: 08-27-99
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***********************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>

#include "cgiutils.h"

char main_config_file[MAX_FILENAME_LENGTH];
char hosts_config_file[MAX_FILENAME_LENGTH];

char log_file[MAX_FILENAME_LENGTH];
char status_file[MAX_FILENAME_LENGTH];

char physical_html_path[MAX_FILENAME_LENGTH];
char physical_images_path[MAX_FILENAME_LENGTH];
char url_html_path[MAX_FILENAME_LENGTH];
char url_docs_path[MAX_FILENAME_LENGTH];
char url_images_path[MAX_FILENAME_LENGTH];
char url_logo_images_path[MAX_FILENAME_LENGTH];
char url_cgibin_path[MAX_FILENAME_LENGTH];


hostgroup *hostgroup_list=NULL;
hoststatus *hoststatus_list=NULL;
host *host_list=NULL;
servicestatus *servicestatus_list=NULL;

extern char *tzname[2];

char *my_strtok_buffer=NULL;
char *original_my_strtok_buffer=NULL;


void reset_cgi_vars(void){

	strcpy(main_config_file,"");
	strcpy(hosts_config_file,"");

	strcpy(url_html_path,"");
	strcpy(url_docs_path,"");
	strcpy(url_images_path,"");
	strcpy(url_cgibin_path,"");

	strcpy(log_file,"");
	strcpy(status_file,"");

	hostgroup_list=NULL;
	hoststatus_list=NULL;
	servicestatus_list=NULL;

	return;
        }


void free_memory(void){

	free_hostgroup_memory();
	free_host_status_memory();
	free_host_memory();
	free_service_status_memory();

	return;
        }


void free_hostgroup_memory(void){
	hostgroup *this_hostgroup;
	hostgroup *next_hostgroup;

	/* free memory for the hostgroup list */
	this_hostgroup=hostgroup_list;
	while(this_hostgroup!=NULL){
		next_hostgroup=this_hostgroup->next;
		free(this_hostgroup);
		this_hostgroup=next_hostgroup;
		}

	return;
        }


void free_host_status_memory(void){
	hoststatus *this_hoststatus;
	hoststatus *next_hoststatus;

	/* free memory for the host status list */
	this_hoststatus=hoststatus_list;
	while(this_hoststatus!=NULL){
		next_hoststatus=this_hoststatus->next;
		free(this_hoststatus);
		this_hoststatus=next_hoststatus;
	        }

	return;
        }


void free_host_memory(void){
	host *this_host;
	host *next_host;

	/* free memory for the host list */
	this_host=host_list;
	while(this_host!=NULL){
		next_host=this_host->next;
		free(this_host);
		this_host=next_host;
	        }

	return;
        }


void free_service_status_memory(void){
	servicestatus *this_svcstatus;
	servicestatus *next_svcstatus;

	/* free memory for the service status list */
	this_svcstatus=servicestatus_list;
	while(this_svcstatus!=NULL){
		next_svcstatus=this_svcstatus->next;
		free(this_svcstatus);
		this_svcstatus=next_svcstatus;
	        }

	return;
        }


int read_cgi_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;

	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		if((strstr(input_buffer,"main_config_file=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strcpy(main_config_file,temp_buffer);
			strip(main_config_file);
		        }
	        }

	fclose(fp);

	if(!strcmp(main_config_file,""))
		return ERROR;
	else
		return OK;
        }



int read_main_config_file(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	FILE *fp;

	
	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		if((strstr(input_buffer,"url_html_path=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strncpy(url_html_path,temp_buffer,sizeof(url_html_path));
			strip(url_html_path);
			if(url_html_path[strlen(url_html_path)-1]!='/')
				strcat(url_html_path,"/");
			sprintf(url_docs_path,"%sdocs/",url_html_path);
			sprintf(url_images_path,"%simages/",url_html_path);
			sprintf(url_logo_images_path,"%srw/",url_images_path);
		        }

		if((strstr(input_buffer,"physical_html_path=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\n");
			strncpy(physical_html_path,temp_buffer,sizeof(physical_html_path));
			strip(physical_html_path);
			if(physical_html_path[strlen(physical_html_path)-1]!='/')
				strcat(physical_html_path,"/");
			sprintf(physical_images_path,"%simages/",physical_html_path);
		        }

		else if((strstr(input_buffer,"url_cgibin_path=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(url_cgibin_path,temp_buffer,sizeof(url_cgibin_path));
			strip(url_cgibin_path);
			if(url_cgibin_path[strlen(url_cgibin_path)-1]!='/')
				strcat(url_cgibin_path,"/");
		        }


		else if((strstr(input_buffer,"log_file=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(log_file,temp_buffer,sizeof(log_file));
			strip(log_file);
		        }

		else if((strstr(input_buffer,"status_file=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(status_file,temp_buffer,sizeof(status_file));
			strip(status_file);
		        }
	
		else if((strstr(input_buffer,"cfg_file=")==input_buffer) && strlen(input_buffer)<MAX_FILENAME_LENGTH){
			temp_buffer=strtok(input_buffer,"=");
			temp_buffer=strtok(NULL,"\x0");
			strncpy(hosts_config_file,temp_buffer,sizeof(hosts_config_file));
			strip(hosts_config_file);
		        }
                }

	fclose(fp);
	return OK;
        }


int read_hosts_config_file(char *filename,int options){
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	hostgroup *new_hostgroup;
	hostgroup *last_hostgroup;
	hostgroup *temp_hostgroup;
	host *new_host;
	host *last_host;
	host *temp_host;
	FILE *fp;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		/* if the hostgroup identifier is found at the start of the line... */
		if((strstr(input_buffer,"hostgroup[")==input_buffer) && (options & READ_HOSTGROUPS)){

			/* add the hostgroup entry to the list in memory */
			new_hostgroup=(hostgroup *)malloc(sizeof(hostgroup));
			if(new_hostgroup!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strcpy(new_hostgroup->group_name,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_hostgroup->group_alias,(temp_buffer==NULL)?"":temp_buffer+1);
				temp_buffer=my_strtok(NULL,";");
				temp_buffer=my_strtok(NULL,"\x0");
				strcpy(new_hostgroup->members,(temp_buffer==NULL)?"":temp_buffer);
				strip(new_hostgroup->members);

				/* add new hostgroup to hostgroup list, sorted by hostgroup name */
				last_hostgroup=hostgroup_list;
				for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
					if(strcmp(new_hostgroup->group_name,temp_hostgroup->group_name)<0){
						new_hostgroup->next=temp_hostgroup;
						if(temp_hostgroup==hostgroup_list)
							hostgroup_list=new_hostgroup;
						else
							last_hostgroup->next=new_hostgroup;
						break;
					        }
					else
						last_hostgroup=temp_hostgroup;
				        }
				if(hostgroup_list==NULL){
					new_hostgroup->next=NULL;
					hostgroup_list=new_hostgroup;
				        }
				else if(temp_hostgroup==NULL){
					new_hostgroup->next=NULL;
					last_hostgroup->next=new_hostgroup;
				        }

			        }
		        }

		/* we found a host definition... */
		else if((strstr(input_buffer,"host[")==input_buffer) && (options & READ_HOSTS)){

			/* add the new host to the list in memory */
			new_host=(host *)malloc(sizeof(host));
			if(new_host!=NULL){
				
				temp_buffer=my_strtok(input_buffer,"[");
				temp_buffer=my_strtok(NULL,"]");
				strcpy(new_host->name,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_host->alias,(temp_buffer==NULL)?"":temp_buffer+1);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_host->address,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_host->parent_host,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_host->host_check_command,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notification_interval=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				strcpy(new_host->notification_period,(temp_buffer==NULL)?"":temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_recovery=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_down=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,";");
				new_host->notify_on_unreachable=(temp_buffer==NULL)?0:atoi(temp_buffer);
				temp_buffer=my_strtok(NULL,"\x0");
				strcpy(new_host->event_handler,(temp_buffer==NULL)?"":temp_buffer);
				strip(new_host->event_handler);
				new_host->number_of_immediate_children=0;
				new_host->max_child_drawing_width=0;

				/* add new host to host list, sorted by host name */
				last_host=host_list;
				for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
					if(strcmp(new_host->name,temp_host->name)<0){
						new_host->next=temp_host;
						if(temp_host==host_list)
							host_list=new_host;
						else
							last_host->next=new_host;
						break;
					        }
					else
						last_host=temp_host;
				        }
				if(host_list==NULL){
					new_host->next=NULL;
					host_list=new_host;
				        }
				else if(temp_host==NULL){
					new_host->next=NULL;
					last_host->next=new_host;
				        }
			         }
		        }

		/* we found a service definition... */
		else if(strstr(input_buffer,"service[")==input_buffer){
		        }

		/* we found a contact definition... */
		else if(strstr(input_buffer,"contact[")==input_buffer){
		        }

		/* we found a command definition... */
		else if(strstr(input_buffer,"command[")==input_buffer){
		        }

		/* we found a timeperiod definition... */
		else if(strstr(input_buffer,"timeperiod[")==input_buffer){
		        }

	        }

	fclose(fp);

	return OK;
        }


int read_status_log(char *filename){
	char input_buffer[MAX_INPUT_BUFFER];
	FILE *fp;
	time_t last_check;
	time_t next_check;
	char host_name[MAX_HOSTNAME_LENGTH];
	char service_description[MAX_SERVICEDESC_LENGTH];
	char service_information[MAX_SERVICEINFO_LENGTH];
	char host_information[MAX_HOSTINFO_LENGTH];
	char status[MAX_STATUS_LENGTH];
	int max_attempts;
	int current_attempt;
	char *temp_buffer;


	fp=fopen(filename,"r");
	if(fp==NULL)
		return ERROR;

	while(read_line(input_buffer,MAX_INPUT_BUFFER,fp)){

	        if(feof(fp))
		        break;

		if(input_buffer==NULL)
		        continue;

		if(strstr(input_buffer,"] HOST;")){
		
			temp_buffer=my_strtok(input_buffer,"]");
			last_check=(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			temp_buffer=my_strtok(NULL,";");
			strcpy(host_name,(temp_buffer==NULL)?"":temp_buffer);

			temp_buffer=my_strtok(NULL,";");
			strcpy(status,(temp_buffer==NULL)?"":temp_buffer);

			temp_buffer=my_strtok(NULL,";");
			strcpy(host_information,(temp_buffer==NULL)?"":temp_buffer);

			add_host_status(host_name,status,host_information,last_check);
		        }

		else if(strstr(input_buffer,"] SERVICE;")){

			temp_buffer=my_strtok(input_buffer,"]");
			last_check=(temp_buffer==NULL)?0L:strtoul(temp_buffer+1,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			temp_buffer=my_strtok(NULL,";");
			strcpy(host_name,(temp_buffer==NULL)?"":temp_buffer);
			
			temp_buffer=my_strtok(NULL,";");
			strcpy(service_description,(temp_buffer==NULL)?"":temp_buffer);

			temp_buffer=my_strtok(NULL,";");
			strcpy(status,(temp_buffer==NULL)?"":temp_buffer);
			
			temp_buffer=my_strtok(NULL,"/");
			current_attempt=(temp_buffer==NULL)?0:atoi(temp_buffer);

			temp_buffer=my_strtok(NULL,";");
			max_attempts=(temp_buffer==NULL)?0:atoi(temp_buffer);
			
			temp_buffer=my_strtok(NULL,";");
			next_check=(temp_buffer==NULL)?0L:strtoul(temp_buffer,NULL,10);

			temp_buffer=my_strtok(NULL,";");
			strcpy(service_information,(temp_buffer==NULL)?"":temp_buffer);

			add_service_status(host_name,service_description,status,current_attempt,max_attempts,service_information,last_check,next_check);
		        }
	        }

	fclose(fp);

	return OK;
        }


void add_host_status(char *host_name, char *status, char *host_information, time_t last_check){
	hoststatus *new_status;
	hoststatus *last_status;
	hoststatus *temp_status;

	/* allocate memory for new host status */
	new_status=(hoststatus *)malloc(sizeof(hoststatus));
	if(new_status==NULL)
		return;

	strcpy(new_status->host_name,host_name);
	if(!strcmp(status,"DOWN"))
		new_status->status=HOST_DOWN;
	else if(!strcmp(status,"UNREACHABLE"))
		new_status->status=HOST_UNREACHABLE;
	else if(!strcmp(status,"PENDING"))
		new_status->status=HOST_PENDING;
	else
		new_status->status=HOST_UP;
	strcpy(new_status->information,host_information);
	new_status->last_check=last_check;

	/* add new host status to list, sorted by host name */

	last_status=hoststatus_list;
	for(temp_status=hoststatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(strcmp(new_status->host_name,temp_status->host_name)<0){
			new_status->next=temp_status;
			if(temp_status==hoststatus_list)
				hoststatus_list=new_status;
			else
				last_status->next=new_status;
			break;
		        }
		else
			last_status=temp_status;
	        }
	if(hoststatus_list==NULL){
		new_status->next=NULL;
		hoststatus_list=new_status;
	        }
	else if(temp_status==NULL){
		new_status->next=NULL;
		last_status->next=new_status;
	        }
	
	return;
        }


void add_service_status(char *host_name, char *service_description, char *status, int current_attempt, int max_attempts, char *service_information, time_t last_check, time_t next_check){
	servicestatus *new_status;
	servicestatus *last_status;
	servicestatus *temp_status;


	/* allocate memory for a new service status entry */
	new_status=(servicestatus *)malloc(sizeof(servicestatus));
	if(new_status==NULL)
		return;

	strcpy(new_status->host_name,host_name);
	strcpy(new_status->description,service_description);
	strcpy(new_status->information,service_information);

	new_status->current_attempt=current_attempt;
	new_status->max_attempts=max_attempts;
	new_status->last_check=last_check;
	new_status->next_check=next_check;

	if(!strcmp(status,"WARNING"))
		new_status->status=SERVICE_WARNING;
	else if(!strcmp(status,"UNKNOWN"))
		new_status->status=SERVICE_UNKNOWN;
	else if(!strcmp(status,"CRITICAL"))
		new_status->status=SERVICE_CRITICAL;
	else if(!strcmp(status,"RECOVERY"))
		new_status->status=SERVICE_RECOVERY;
	else if(!strcmp(status,"HOST DOWN"))
		new_status->status=SERVICE_HOST_DOWN;
	else if(!strcmp(status,"UNREACHABLE"))
		new_status->status=SERVICE_UNREACHABLE;
	else if(!strcmp(status,"PENDING"))
		new_status->status=SERVICE_PENDING;
	else
		new_status->status=SERVICE_OK;

	/* add new service status to front of list */
	last_status=servicestatus_list;
	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(strcmp(new_status->host_name,temp_status->host_name)<0){
			new_status->next=temp_status;
			if(temp_status==servicestatus_list)
				servicestatus_list=new_status;
			else
				last_status->next=new_status;
			break;
		        }
		else
			last_status=temp_status;
	        }
	if(servicestatus_list==NULL){
		new_status->next=NULL;
		servicestatus_list=new_status;
	        }
	else if(temp_status==NULL){
		new_status->next=NULL;
		last_status->next=new_status;
	        }

	return;
        }


int get_host_count(char *host_name, int type){
	hoststatus *temp_status;
	int count=0;

	for(temp_status=hoststatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(temp_status->status & type){
			if(host_name==NULL)
				count++;
			else if(!strcmp(host_name,temp_status->host_name))
				count++;
		        }
	        }

	return count;
        }


int get_service_count(char *host_name, int type){
	servicestatus *temp_status;
	int count=0;

	for(temp_status=servicestatus_list;temp_status!=NULL;temp_status=temp_status->next){
		if(temp_status->status & type){
			if(host_name==NULL)
				count++;
			else if(!strcmp(host_name,temp_status->host_name))
				count++;
		        }
	        }

	return count;
        }


int is_member_of_hostgroup(hostgroup *hg, char *host_name){
	char members[MAX_INPUT_BUFFER];
	char *temp_buffer;

	if(hg==NULL)
		return FALSE;

	strncpy(members,hg->members,sizeof(members)-1);
	members[sizeof(members)-1]='\x0';

	for(temp_buffer=(char *)my_strtok(members,",");temp_buffer!=NULL;temp_buffer=(char *)my_strtok(NULL,","))
		if(!strcmp(temp_buffer,host_name))
			return TRUE;

	return FALSE;
        }


/* recursively strips newline characters, spaces, and carriage returns from the end of a string */
void strip(char *buffer){
	int x;

	x=strlen(buffer);

	if(!x)
		return;
	else
		x--;

	if(buffer[x]==' ' || buffer[x]=='\n' || buffer[x]=='\r' || buffer[x]=='\t' || buffer[x]==13)
		buffer[x]='\x0';

	else
		return;

	strip(buffer);

	return;
        }


/* reads a line of text from a file and strips it clean */
char * read_raw_line(char *buffer, int maxlength, FILE *fp){
	char *result;

	/* clear the buffer */
	strcpy(buffer,"");

	/* read in one line of text */
	result=fgets(buffer,maxlength-1,fp);

	/* if we were able to read something, strip extra characters from the end */
	if(result){
		buffer[maxlength-1]='\x0';
		strip(buffer);
	        }

	/* return the number of characters read */
	return result;
        }


/* read a line of text from a file, skipping comments and empty lines */
char * read_line(char *buffer, int maxlength, FILE *fp){
	char *result;
	int keep_line=TRUE;

	while((result=read_raw_line(buffer,maxlength,fp))){

		if(buffer[0]=='#')
			keep_line=FALSE;
		else if(buffer[0]=='\r')
			keep_line=FALSE;
		else if(buffer[0]=='\n')
			keep_line=FALSE;
		else if(buffer[0]=='\x0')
			keep_line=FALSE;
		else
			keep_line=TRUE;

		if(keep_line==TRUE)
			break;
	        }

	return result;
        }


hostgroup * find_hostgroup(char *hostgroup_name){
	hostgroup *temp_hostgroup;

	for(temp_hostgroup=hostgroup_list;temp_hostgroup!=NULL;temp_hostgroup=temp_hostgroup->next){
		if(!strcmp(hostgroup_name,temp_hostgroup->group_name))
			return temp_hostgroup;
	        }
	
	return NULL;
        }



void get_time_string(unsigned long *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};

	if(raw_time==NULL)
		time(&t);
	else 
		t=(time_t)(*raw_time);

	tm_ptr=localtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

	snprintf(buffer,buffer_length-1,"%s %s %d %02d:%02d:%02d %s %d",weekdays[tm_ptr->tm_wday],months[tm_ptr->tm_mon],day,hour,minute,second,tzname[1],year);
	buffer[buffer_length-1]='\x0';

	return;
        }


void get_expire_time_string(unsigned long *raw_time,char *buffer,int buffer_length){
	time_t t;
	struct tm *tm_ptr;
	int day;
	int hour;
	int minute;
	int second;
	int year;
	char *weekdays[7]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	char *months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sept","Oct","Nov","Dec"};

	if(raw_time==NULL)
		time(&t);
	else 
		t=(time_t)(*raw_time);

	tm_ptr=gmtime(&t);

	hour=tm_ptr->tm_hour;
	minute=tm_ptr->tm_min;
	second=tm_ptr->tm_sec;
	day=tm_ptr->tm_mday;
	year=tm_ptr->tm_year+1900;

	snprintf(buffer,buffer_length-1,"%s, %d %s %d %02d:%02d:%02d GMT",weekdays[tm_ptr->tm_wday],day,months[tm_ptr->tm_mon],year,hour,minute,second);
	buffer[buffer_length-1]='\x0';

	return;
        }


/* determines whether or not a specific host is an immediate child of another host */
int is_immediate_child_of_host(host *parent_host,host *child_host){

	if(parent_host==NULL){
		if(!strcmp(child_host->parent_host,""))
			return 1;
		else
			return 0;
	        }

	else if(!strcmp(child_host->parent_host,parent_host->name))
		return 1;

	return 0;
	}


/* returns a count of the immediate children for a given host */
int number_of_immediate_children(host *hst){
	int children=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host))
			children++;
		}

	return children;
	}


/* returns a count of the total children for a given host */
int number_of_children(host *hst){
	int children=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host))
			children+=number_of_children(temp_host)+1;
		}

	return children;
	}


/* get the max depth of children hosts for a given host */
int get_max_host_child_depth(host *hst){
	int max_depth=0;
	int temp_depth=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_child_of_host(hst,temp_host)){
			temp_depth=get_max_host_child_depth(temp_host);
			if(temp_depth>max_depth)
				max_depth=temp_depth;
		        }
	        }

	return max_depth+1;
        }


/* determines whether or not a specific host is an immediate parent of another host */
int is_immediate_parent_of_host(host *child_host,host *parent_host){

	if(child_host==NULL || parent_host==NULL)
		return 0;

	else if(!strcmp(child_host->parent_host,parent_host->name))
		return 1;

	return 0;
	}


/* get the total number of parent hosts for a given host */
int number_of_parents(host *hst){
	int parents=0;
	host *temp_host;

	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
		if(is_immediate_parent_of_host(hst,temp_host)){
			parents+=number_of_parents(temp_host)+1;
			break;
		        }
	        }

	return parents;
        }


/* fix the problem with strtok() skipping empty options between tokens */	
char *my_strtok(char *buffer,char *tokens){
	char *token_position;
	char *sequence_head;

	if(buffer!=NULL){
		if(original_my_strtok_buffer!=NULL)
			free(original_my_strtok_buffer);
		my_strtok_buffer=malloc(strlen(buffer)+1);
		if(my_strtok_buffer==NULL)
			return NULL;
		original_my_strtok_buffer=my_strtok_buffer;
		strcpy(my_strtok_buffer,buffer);
	        }
	
	sequence_head=my_strtok_buffer;

	if(sequence_head[0]=='\x0')
		return NULL;
	
	token_position=index(my_strtok_buffer,tokens[0]);

	if(token_position==NULL){
		my_strtok_buffer=index(my_strtok_buffer,'\x0');
		return sequence_head;
	        }

	token_position[0]='\x0';
	my_strtok_buffer=token_position+1;

	return sequence_head;

        }

