/*
 ** check_ssh.c
 ** 
 ** Made by (Remi PAULMIER)
 ** Login   <remi@sinfomic.fr>
 ** 
 ** Started on  Fri Jul  9 09:18:23 1999 Remi PAULMIER
 ** Last update Thu Jul 22 12:50:04 1999 remi paulmier
 ** compile it with "gcc -Wall -O2 check_ssh.c -o check_ssh"
 */

#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>


#define DEFAULT_TIMEOUT	10	/* connection timeout in seconds */
#define SSH_DFL_PORT    22
#define BUFF_SZ         256

short port;

char    *ssh_resolve(hostname)
char    *hostname;
{
  struct hostent *host;

  host = gethostbyname(hostname);
  if (!host) {
    herror(hostname);
    exit (2);
  }
  return (host->h_addr);
}

int     ssh_connect(haddr, hport)
char    *haddr;
short   hport;
{
  int   s;
  struct sockaddr_in addr;
  int   addrlen;
  char  *buffer;
  char  *ssh_proto;
  char  *ssh_server;
  
  addrlen = sizeof(addr);
  memset(&addr, 0, addrlen);
  addr.sin_port = htons(hport);
  addr.sin_family = AF_INET;
  bcopy(haddr, (void *) &addr.sin_addr.s_addr, 4);
  
  s = socket(AF_INET, SOCK_STREAM, 0);
  if (!s) {
    printf("socket(): %s\n", strerror(errno));
    exit (2);
  }

  if (connect(s, (struct sockaddr *) &addr, addrlen)) {
    printf("connect(): %s\n", strerror(errno));
    exit (2);
  }

  buffer = (char *) malloc(BUFF_SZ + 1);
  memset(buffer, 0, BUFF_SZ + 1);
  recv(s, buffer, BUFF_SZ, 0);
  if (strncmp(buffer, "SSH", 3)) {
    printf("Server answer: %s", buffer);
    exit (2);
  }
  else {
    ssh_proto = strtok(buffer + 4, "-");
    ssh_server = strtok(0, "-");
    printf("SSH ok - protocol version %s - server version %s", 
           ssh_proto,
           ssh_server);
    exit (0);
  }
}


void alarm_handler(int sig){

	printf("Socket timeout after %d seconds\n",DEFAULT_TIMEOUT);

	exit(2);
        }


int     main(argc, argv)
int     argc;
char    **argv;
{

  /* initialize alarm signal handling */
  signal(SIGALRM,alarm_handler);

  /* set socket timeout */
  alarm(DEFAULT_TIMEOUT);

  switch (argc) {
  default:
    printf("Incorrect number of arguments supplied\n\n"
           "SSH plugin for NetSaint\n"
           "Copyright (c) 1999 Remi Paulmier (remi@sinfomic.fr)\n"
           "Last Modified: 07-22-1999\n"
           "License: GPL\n\n"
           "Usage: %s <dst_addr> [port]\n\n"
           "by default, port is %d\n",
           argv[0],
           SSH_DFL_PORT);
    exit (2);
    
  case 2:
    port = SSH_DFL_PORT;
    ssh_connect(ssh_resolve(argv[1]), port);
    break;
    
  case 3:
    port = atoi(argv[2]);
    ssh_connect(ssh_resolve(argv[1]), port);
    break;
  }

  /* reset the alarm */
  alarm(0);

  return (0);
}



/* end of check_ssh.c */
