/******************************************************************************
 *
 * CHECK_SWAP.C
 *
 * Program: Process plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)
 *
 * Last Modified: 18 May 1999
 *
 * Command line: check_swap <wsize> <csize> [command_name]
 *
 * Description:
 *
 * This plugin will check for processes whose total image size exceeds the 
 * warning or critical thresholds given on the command line.  With no
 * command_name, everything that shows up on ps is evaluated.  Otherwise,
 * only jobs with the command_name given are examined.  This program is
 * particularly useful if you have to run a piece of commercial software
 * that has a memory leak.  With it you can shut down ad restart the processes
 * whenever the program threatens to take over your system.
 *****************************************************************************/

#include "../common/config.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	256

#define MYMAX(x,y) ((x)>(y) ? (x) : (y))



int main(int argc, char **argv){
	int wswap, cswap, pct;
	float tswap, uswap;
	int result=STATE_OK;
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char *temp_buffer;
	char frmt_str[32];
	char status[MAX_INPUT_BUFFER];

	if(argc !=3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("Process plugin for NetSaint\n");
		printf("Copyright (c) 1999 Karl DeBisschop (kdebiss@alum.mit.edu)\n");
		printf("Last Modified: 18 May 1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <wswap> <cswap>\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" <wsize>     = Consumed swap percentage necessary to cause a WARNING state\n");
		printf(" <csize>     = Consumed swap percentage necessary to cause a CRITICAL state\n");
		printf("\n");
		printf("This plugin will check all of the swap partitions and return an\n");
		printf("error if the number is above either of the thresholds given.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the number of procs use as warning and critical thresholds */
	wswap=atoi(argv[1]);
	cswap=atoi(argv[2]);

	/* run the command */
	fp=popen(SWAP_COMMAND,"r");
	if(fp==NULL){
		return STATE_UNKNOWN;
	        }
	
	sprintf(frmt_str, "%s", "");
	/* read 1st line */
	fgets(input_buffer,MAX_INPUT_BUFFER-1,fp); 
	temp_buffer=strtok(input_buffer," \n");
	while (temp_buffer) {
	  if (strstr(temp_buffer,"blocks"))
	    sprintf(frmt_str,"%s %s",frmt_str,"%f");
	  else if (strstr(temp_buffer,"free"))
	    sprintf(frmt_str,"%s %s",frmt_str,"%f");
	  else
	    sprintf(frmt_str,"%s %s",frmt_str,"%*s");
	  temp_buffer=strtok(NULL," \n");
	}

	sprintf(status,"%s","Swap used:");
	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)) {
	  sscanf(input_buffer, frmt_str, &tswap, &uswap);
	  pct = 100 * (1.0 - uswap/tswap);
	  sprintf(status,"%s %2d%%",status,pct);
	  if (pct > cswap)
	    result = STATE_CRITICAL;
	  else if (pct > wswap)
	    result = MYMAX(result, STATE_WARNING);
	}
	  
	/* close the temp file */
	fclose(fp);

	if(result==STATE_OK)
	  printf("Swap ok - %s\n",status);
	else if(result==STATE_CRITICAL)
	  printf("CRITICAL - %s\n",status);
	else if(result==STATE_WARNING)
	  printf("WARNING - %s\n",status);
	else if(result==STATE_UNKNOWN)
	  printf("Unable to read output\n");
	else {
	  result = STATE_UNKNOWN;
	  printf("UNKNOWN - %s\n",status);
	}

	return result;
        }
